/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This code was generated by https://code.google.com/p/google-apis-client-generator/
 * (build: 2013-06-26 16:27:34 UTC)
 * on 2013-07-15 at 20:00:32 UTC 
 * Modify at your own risk.
 */

package com.google.api.services.admin.directory;

/**
 * Service definition for Directory (directory_v1).
 *
 * <p>
 * Apps Directory API lets you view and manage enterprise resources like user, groups, OrgUnit, devices.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/admin-sdk/directory/" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link DirectoryRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Directory extends com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    com.google.api.client.util.Preconditions.checkState(
        com.google.api.client.googleapis.GoogleUtils.MAJOR_VERSION == 1 &&
        com.google.api.client.googleapis.GoogleUtils.MINOR_VERSION == 14,
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.14 of google-api-client to run version " +
        "1.14.2-beta of the Admin Directory API library.", com.google.api.client.googleapis.GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "admin/directory/v1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   */
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport, which should normally be:
   *        <ul>
   *        <li>Google App Engine:
   *        {@code com.google.api.client.extensions.appengine.http.UrlFetchTransport}</li>
   *        <li>Android: {@code newCompatibleTransport} from
   *        {@code com.google.api.client.extensions.android.http.AndroidHttp}</li>
   *        <li>Java: {@link com.google.api.client.googleapis.javanet.GoogleNetHttpTransport#newTrustedTransport()}
   *        </li>
   *        </ul>
   * @param jsonFactory JSON factory, which may be:
   *        <ul>
   *        <li>Jackson: {@code com.google.api.client.json.jackson2.JacksonFactory}</li>
   *        <li>Google GSON: {@code com.google.api.client.json.gson.GsonFactory}</li>
   *        <li>Android Honeycomb or higher:
   *        {@code com.google.api.client.extensions.android.json.AndroidJsonFactory}</li>
   *        </ul>
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Directory(com.google.api.client.http.HttpTransport transport, com.google.api.client.json.JsonFactory jsonFactory,
      com.google.api.client.http.HttpRequestInitializer httpRequestInitializer) {
    this(new Builder(transport, jsonFactory, httpRequestInitializer));
  }

  /**
   * @param builder builder
   */
  Directory(Builder builder) {
    super(builder);
  }

  @Override
  protected void initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Chromeosdevices collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Chromeosdevices.List request = admin.chromeosdevices().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Chromeosdevices chromeosdevices() {
    return new Chromeosdevices();
  }

  /**
   * The "chromeosdevices" collection of methods.
   */
  public class Chromeosdevices {

    /**
     * Retrieve Chrome OS Device
     *
     * Create a request for the method "chromeosdevices.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param deviceId Immutable id of Chrome OS Device
     * @return the request
     */
    public Get get(java.lang.String customerId, java.lang.String deviceId) throws java.io.IOException {
      Get result = new Get(customerId, deviceId);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.ChromeOsDevice> {

      private static final String REST_PATH = "customer/{customerId}/devices/chromeos/{deviceId}";

      /**
       * Retrieve Chrome OS Device
       *
       * Create a request for the method "chromeosdevices.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param deviceId Immutable id of Chrome OS Device
       * @since 1.13
       */
      protected Get(java.lang.String customerId, java.lang.String deviceId) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.ChromeOsDevice.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.deviceId = com.google.api.client.util.Preconditions.checkNotNull(deviceId, "Required parameter deviceId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(java.lang.String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(java.lang.String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(java.lang.String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(java.lang.String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(java.lang.String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(java.lang.String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Get setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Chrome OS Device */
      @com.google.api.client.util.Key
      private java.lang.String deviceId;

      /** Immutable id of Chrome OS Device
       */
      public java.lang.String getDeviceId() {
        return deviceId;
      }

      /** Immutable id of Chrome OS Device */
      public Get setDeviceId(java.lang.String deviceId) {
        this.deviceId = deviceId;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private java.lang.String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public java.lang.String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public Get setProjection(java.lang.String projection) {
        this.projection = projection;
        return this;
      }

      @Override
      public Get set(String parameterName, Object value) {
        return (Get) super.set(parameterName, value);
      }
    }
    /**
     * Retrieve all Chrome OS Devices of a customer (paginated)
     *
     * Create a request for the method "chromeosdevices.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @return the request
     */
    public List list(java.lang.String customerId) throws java.io.IOException {
      List result = new List(customerId);
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.ChromeOsDevices> {

      private static final String REST_PATH = "customer/{customerId}/devices/chromeos";

      /**
       * Retrieve all Chrome OS Devices of a customer (paginated)
       *
       * Create a request for the method "chromeosdevices.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @since 1.13
       */
      protected List(java.lang.String customerId) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.ChromeOsDevices.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(java.lang.String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(java.lang.String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(java.lang.String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(java.lang.String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(java.lang.String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(java.lang.String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public List setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Column to use for sorting results */
      @com.google.api.client.util.Key
      private java.lang.String orderBy;

      /** Column to use for sorting results
       */
      public java.lang.String getOrderBy() {
        return orderBy;
      }

      /** Column to use for sorting results */
      public List setOrderBy(java.lang.String orderBy) {
        this.orderBy = orderBy;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private java.lang.String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public java.lang.String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public List setProjection(java.lang.String projection) {
        this.projection = projection;
        return this;
      }

      /** Maximum number of results to return. Default is 100 */
      @com.google.api.client.util.Key
      private java.lang.Integer maxResults;

      /** Maximum number of results to return. Default is 100

     [minimum: 1]
       */
      public java.lang.Integer getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. Default is 100 */
      public List setMaxResults(java.lang.Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** Token to specify next page in the list */
      @com.google.api.client.util.Key
      private java.lang.String pageToken;

      /** Token to specify next page in the list
       */
      public java.lang.String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page in the list */
      public List setPageToken(java.lang.String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**
       * Whether to return results in ascending or descending order. Only of use when orderBy is
       * also used
       */
      @com.google.api.client.util.Key
      private java.lang.String sortOrder;

      /** Whether to return results in ascending or descending order. Only of use when orderBy is also used
       */
      public java.lang.String getSortOrder() {
        return sortOrder;
      }

      /**
       * Whether to return results in ascending or descending order. Only of use when orderBy is
       * also used
       */
      public List setSortOrder(java.lang.String sortOrder) {
        this.sortOrder = sortOrder;
        return this;
      }

      /**
       * Search string in the format given at
       * http://support.google.com/chromeos/a/bin/answer.py?hl=en=1698333
       */
      @com.google.api.client.util.Key
      private java.lang.String query;

      /** Search string in the format given at
     http://support.google.com/chromeos/a/bin/answer.py?hl=en=1698333
       */
      public java.lang.String getQuery() {
        return query;
      }

      /**
       * Search string in the format given at
       * http://support.google.com/chromeos/a/bin/answer.py?hl=en=1698333
       */
      public List setQuery(java.lang.String query) {
        this.query = query;
        return this;
      }

      @Override
      public List set(String parameterName, Object value) {
        return (List) super.set(parameterName, value);
      }
    }
    /**
     * Update Chrome OS Device. This method supports patch semantics.
     *
     * Create a request for the method "chromeosdevices.patch".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param deviceId Immutable id of Chrome OS Device
     * @param content the {@link com.google.api.services.admin.directory.model.ChromeOsDevice}
     * @return the request
     */
    public Patch patch(java.lang.String customerId, java.lang.String deviceId, com.google.api.services.admin.directory.model.ChromeOsDevice content) throws java.io.IOException {
      Patch result = new Patch(customerId, deviceId, content);
      initialize(result);
      return result;
    }

    public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.ChromeOsDevice> {

      private static final String REST_PATH = "customer/{customerId}/devices/chromeos/{deviceId}";

      /**
       * Update Chrome OS Device. This method supports patch semantics.
       *
       * Create a request for the method "chromeosdevices.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param deviceId Immutable id of Chrome OS Device
       * @param content the {@link com.google.api.services.admin.directory.model.ChromeOsDevice}
       * @since 1.13
       */
      protected Patch(java.lang.String customerId, java.lang.String deviceId, com.google.api.services.admin.directory.model.ChromeOsDevice content) {
        super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.ChromeOsDevice.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.deviceId = com.google.api.client.util.Preconditions.checkNotNull(deviceId, "Required parameter deviceId must be specified.");
      }

      @Override
      public Patch setAlt(java.lang.String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(java.lang.String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(java.lang.String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(java.lang.String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(java.lang.String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(java.lang.String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Patch setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Chrome OS Device */
      @com.google.api.client.util.Key
      private java.lang.String deviceId;

      /** Immutable id of Chrome OS Device
       */
      public java.lang.String getDeviceId() {
        return deviceId;
      }

      /** Immutable id of Chrome OS Device */
      public Patch setDeviceId(java.lang.String deviceId) {
        this.deviceId = deviceId;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private java.lang.String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public java.lang.String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public Patch setProjection(java.lang.String projection) {
        this.projection = projection;
        return this;
      }

      @Override
      public Patch set(String parameterName, Object value) {
        return (Patch) super.set(parameterName, value);
      }
    }
    /**
     * Update Chrome OS Device
     *
     * Create a request for the method "chromeosdevices.update".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param deviceId Immutable id of Chrome OS Device
     * @param content the {@link com.google.api.services.admin.directory.model.ChromeOsDevice}
     * @return the request
     */
    public Update update(java.lang.String customerId, java.lang.String deviceId, com.google.api.services.admin.directory.model.ChromeOsDevice content) throws java.io.IOException {
      Update result = new Update(customerId, deviceId, content);
      initialize(result);
      return result;
    }

    public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.ChromeOsDevice> {

      private static final String REST_PATH = "customer/{customerId}/devices/chromeos/{deviceId}";

      /**
       * Update Chrome OS Device
       *
       * Create a request for the method "chromeosdevices.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param deviceId Immutable id of Chrome OS Device
       * @param content the {@link com.google.api.services.admin.directory.model.ChromeOsDevice}
       * @since 1.13
       */
      protected Update(java.lang.String customerId, java.lang.String deviceId, com.google.api.services.admin.directory.model.ChromeOsDevice content) {
        super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.ChromeOsDevice.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.deviceId = com.google.api.client.util.Preconditions.checkNotNull(deviceId, "Required parameter deviceId must be specified.");
      }

      @Override
      public Update setAlt(java.lang.String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(java.lang.String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(java.lang.String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(java.lang.String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(java.lang.String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(java.lang.String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Update setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Chrome OS Device */
      @com.google.api.client.util.Key
      private java.lang.String deviceId;

      /** Immutable id of Chrome OS Device
       */
      public java.lang.String getDeviceId() {
        return deviceId;
      }

      /** Immutable id of Chrome OS Device */
      public Update setDeviceId(java.lang.String deviceId) {
        this.deviceId = deviceId;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private java.lang.String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public java.lang.String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public Update setProjection(java.lang.String projection) {
        this.projection = projection;
        return this;
      }

      @Override
      public Update set(String parameterName, Object value) {
        return (Update) super.set(parameterName, value);
      }
    }

  }

  /**
   * An accessor for creating requests from the Groups collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Groups.List request = admin.groups().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Groups groups() {
    return new Groups();
  }

  /**
   * The "groups" collection of methods.
   */
  public class Groups {

    /**
     * Delete Group
     *
     * Create a request for the method "groups.delete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @return the request
     */
    public Delete delete(java.lang.String groupKey) throws java.io.IOException {
      Delete result = new Delete(groupKey);
      initialize(result);
      return result;
    }

    public class Delete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "groups/{groupKey}";

      /**
       * Delete Group
       *
       * Create a request for the method "groups.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @since 1.13
       */
      protected Delete(java.lang.String groupKey) {
        super(Directory.this, "DELETE", REST_PATH, null, Void.class);
        this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public Delete setAlt(java.lang.String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(java.lang.String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(java.lang.String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(java.lang.String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(java.lang.String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(java.lang.String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private java.lang.String groupKey;

      /** Email or immutable Id of the group
       */
      public java.lang.String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public Delete setGroupKey(java.lang.String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      @Override
      public Delete set(String parameterName, Object value) {
        return (Delete) super.set(parameterName, value);
      }
    }
    /**
     * Retrieve Group
     *
     * Create a request for the method "groups.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @return the request
     */
    public Get get(java.lang.String groupKey) throws java.io.IOException {
      Get result = new Get(groupKey);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.Group> {

      private static final String REST_PATH = "groups/{groupKey}";

      /**
       * Retrieve Group
       *
       * Create a request for the method "groups.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @since 1.13
       */
      protected Get(java.lang.String groupKey) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Group.class);
        this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(java.lang.String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(java.lang.String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(java.lang.String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(java.lang.String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(java.lang.String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(java.lang.String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private java.lang.String groupKey;

      /** Email or immutable Id of the group
       */
      public java.lang.String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public Get setGroupKey(java.lang.String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      @Override
      public Get set(String parameterName, Object value) {
        return (Get) super.set(parameterName, value);
      }
    }
    /**
     * Create Group
     *
     * Create a request for the method "groups.insert".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param content the {@link com.google.api.services.admin.directory.model.Group}
     * @return the request
     */
    public Insert insert(com.google.api.services.admin.directory.model.Group content) throws java.io.IOException {
      Insert result = new Insert(content);
      initialize(result);
      return result;
    }

    public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.Group> {

      private static final String REST_PATH = "groups";

      /**
       * Create Group
       *
       * Create a request for the method "groups.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param content the {@link com.google.api.services.admin.directory.model.Group}
       * @since 1.13
       */
      protected Insert(com.google.api.services.admin.directory.model.Group content) {
        super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.Group.class);
        checkRequiredParameter(content, "content");
        checkRequiredParameter(content.getEmail(), "Group.getEmail()");
      }

      @Override
      public Insert setAlt(java.lang.String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(java.lang.String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(java.lang.String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(java.lang.String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(java.lang.String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(java.lang.String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

      @Override
      public Insert set(String parameterName, Object value) {
        return (Insert) super.set(parameterName, value);
      }
    }
    /**
     * Retrieve all groups in a domain (paginated)
     *
     * Create a request for the method "groups.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() throws java.io.IOException {
      List result = new List();
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.Groups> {

      private static final String REST_PATH = "groups";

      /**
       * Retrieve all groups in a domain (paginated)
       *
       * Create a request for the method "groups.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected List() {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Groups.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(java.lang.String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(java.lang.String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(java.lang.String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(java.lang.String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(java.lang.String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(java.lang.String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /**
       * Immutable id of the Google Apps account. In case of multi-domain, to fetch all groups for a
       * customer, fill this field instead of domain.
       */
      @com.google.api.client.util.Key
      private java.lang.String customer;

      /** Immutable id of the Google Apps account. In case of multi-domain, to fetch all groups for a
     customer, fill this field instead of domain.
       */
      public java.lang.String getCustomer() {
        return customer;
      }

      /**
       * Immutable id of the Google Apps account. In case of multi-domain, to fetch all groups for a
       * customer, fill this field instead of domain.
       */
      public List setCustomer(java.lang.String customer) {
        this.customer = customer;
        return this;
      }

      /** Token to specify next page in the list */
      @com.google.api.client.util.Key
      private java.lang.String pageToken;

      /** Token to specify next page in the list
       */
      public java.lang.String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page in the list */
      public List setPageToken(java.lang.String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**
       * Name of the domain. Fill this field to get groups from only this domain. To return all
       * groups in a multi-domain fill customer field instead.
       */
      @com.google.api.client.util.Key
      private java.lang.String domain;

      /** Name of the domain. Fill this field to get groups from only this domain. To return all groups in a
     multi-domain fill customer field instead.
       */
      public java.lang.String getDomain() {
        return domain;
      }

      /**
       * Name of the domain. Fill this field to get groups from only this domain. To return all
       * groups in a multi-domain fill customer field instead.
       */
      public List setDomain(java.lang.String domain) {
        this.domain = domain;
        return this;
      }

      /** Maximum number of results to return. Default is 200 */
      @com.google.api.client.util.Key
      private java.lang.Integer maxResults;

      /** Maximum number of results to return. Default is 200

     [minimum: 1]
       */
      public java.lang.Integer getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. Default is 200 */
      public List setMaxResults(java.lang.Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /**
       * Email or immutable Id of the user if only those groups are to be listed, the given user is
       * a member of. If Id, it should match with id of user object
       */
      @com.google.api.client.util.Key
      private java.lang.String userKey;

      /** Email or immutable Id of the user if only those groups are to be listed, the given user is a member
     of. If Id, it should match with id of user object
       */
      public java.lang.String getUserKey() {
        return userKey;
      }

      /**
       * Email or immutable Id of the user if only those groups are to be listed, the given user is
       * a member of. If Id, it should match with id of user object
       */
      public List setUserKey(java.lang.String userKey) {
        this.userKey = userKey;
        return this;
      }

      @Override
      public List set(String parameterName, Object value) {
        return (List) super.set(parameterName, value);
      }
    }
    /**
     * Update Group. This method supports patch semantics.
     *
     * Create a request for the method "groups.patch".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
     * @param content the {@link com.google.api.services.admin.directory.model.Group}
     * @return the request
     */
    public Patch patch(java.lang.String groupKey, com.google.api.services.admin.directory.model.Group content) throws java.io.IOException {
      Patch result = new Patch(groupKey, content);
      initialize(result);
      return result;
    }

    public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.Group> {

      private static final String REST_PATH = "groups/{groupKey}";

      /**
       * Update Group. This method supports patch semantics.
       *
       * Create a request for the method "groups.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
       * @param content the {@link com.google.api.services.admin.directory.model.Group}
       * @since 1.13
       */
      protected Patch(java.lang.String groupKey, com.google.api.services.admin.directory.model.Group content) {
        super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.Group.class);
        this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public Patch setAlt(java.lang.String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(java.lang.String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(java.lang.String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(java.lang.String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(java.lang.String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(java.lang.String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      @com.google.api.client.util.Key
      private java.lang.String groupKey;

      /** Email or immutable Id of the group. If Id, it should match with id of group object
       */
      public java.lang.String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      public Patch setGroupKey(java.lang.String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      @Override
      public Patch set(String parameterName, Object value) {
        return (Patch) super.set(parameterName, value);
      }
    }
    /**
     * Update Group
     *
     * Create a request for the method "groups.update".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
     * @param content the {@link com.google.api.services.admin.directory.model.Group}
     * @return the request
     */
    public Update update(java.lang.String groupKey, com.google.api.services.admin.directory.model.Group content) throws java.io.IOException {
      Update result = new Update(groupKey, content);
      initialize(result);
      return result;
    }

    public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.Group> {

      private static final String REST_PATH = "groups/{groupKey}";

      /**
       * Update Group
       *
       * Create a request for the method "groups.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
       * @param content the {@link com.google.api.services.admin.directory.model.Group}
       * @since 1.13
       */
      protected Update(java.lang.String groupKey, com.google.api.services.admin.directory.model.Group content) {
        super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.Group.class);
        this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public Update setAlt(java.lang.String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(java.lang.String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(java.lang.String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(java.lang.String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(java.lang.String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(java.lang.String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      @com.google.api.client.util.Key
      private java.lang.String groupKey;

      /** Email or immutable Id of the group. If Id, it should match with id of group object
       */
      public java.lang.String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      public Update setGroupKey(java.lang.String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      @Override
      public Update set(String parameterName, Object value) {
        return (Update) super.set(parameterName, value);
      }
    }

    /**
     * An accessor for creating requests from the Aliases collection.
     *
     * <p>The typical use is:</p>
     * <pre>
     *   {@code Directory admin = new Directory(...);}
     *   {@code Directory.Aliases.List request = admin.aliases().list(parameters ...)}
     * </pre>
     *
     * @return the resource collection
     */
    public Aliases aliases() {
      return new Aliases();
    }

    /**
     * The "aliases" collection of methods.
     */
    public class Aliases {

      /**
       * Remove a alias for the group
       *
       * Create a request for the method "aliases.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
       *
       * @param groupKey Email or immutable Id of the group
       * @param alias The alias to be removed
       * @return the request
       */
      public Delete delete(java.lang.String groupKey, java.lang.String alias) throws java.io.IOException {
        Delete result = new Delete(groupKey, alias);
        initialize(result);
        return result;
      }

      public class Delete extends DirectoryRequest<Void> {

        private static final String REST_PATH = "groups/{groupKey}/aliases/{alias}";

        /**
         * Remove a alias for the group
         *
         * Create a request for the method "aliases.delete".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
         * Delete#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
         * be called to initialize this instance immediately after invoking the constructor. </p>
         *
         * @param groupKey Email or immutable Id of the group
         * @param alias The alias to be removed
         * @since 1.13
         */
        protected Delete(java.lang.String groupKey, java.lang.String alias) {
          super(Directory.this, "DELETE", REST_PATH, null, Void.class);
          this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
          this.alias = com.google.api.client.util.Preconditions.checkNotNull(alias, "Required parameter alias must be specified.");
        }

        @Override
        public Delete setAlt(java.lang.String alt) {
          return (Delete) super.setAlt(alt);
        }

        @Override
        public Delete setFields(java.lang.String fields) {
          return (Delete) super.setFields(fields);
        }

        @Override
        public Delete setKey(java.lang.String key) {
          return (Delete) super.setKey(key);
        }

        @Override
        public Delete setOauthToken(java.lang.String oauthToken) {
          return (Delete) super.setOauthToken(oauthToken);
        }

        @Override
        public Delete setPrettyPrint(java.lang.Boolean prettyPrint) {
          return (Delete) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Delete setQuotaUser(java.lang.String quotaUser) {
          return (Delete) super.setQuotaUser(quotaUser);
        }

        @Override
        public Delete setUserIp(java.lang.String userIp) {
          return (Delete) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the group */
        @com.google.api.client.util.Key
        private java.lang.String groupKey;

        /** Email or immutable Id of the group
         */
        public java.lang.String getGroupKey() {
          return groupKey;
        }

        /** Email or immutable Id of the group */
        public Delete setGroupKey(java.lang.String groupKey) {
          this.groupKey = groupKey;
          return this;
        }

        /** The alias to be removed */
        @com.google.api.client.util.Key
        private java.lang.String alias;

        /** The alias to be removed
         */
        public java.lang.String getAlias() {
          return alias;
        }

        /** The alias to be removed */
        public Delete setAlias(java.lang.String alias) {
          this.alias = alias;
          return this;
        }

        @Override
        public Delete set(String parameterName, Object value) {
          return (Delete) super.set(parameterName, value);
        }
      }
      /**
       * Add a alias for the group
       *
       * Create a request for the method "aliases.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
       *
       * @param groupKey Email or immutable Id of the group
       * @param content the {@link com.google.api.services.admin.directory.model.Alias}
       * @return the request
       */
      public Insert insert(java.lang.String groupKey, com.google.api.services.admin.directory.model.Alias content) throws java.io.IOException {
        Insert result = new Insert(groupKey, content);
        initialize(result);
        return result;
      }

      public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.Alias> {

        private static final String REST_PATH = "groups/{groupKey}/aliases";

        /**
         * Add a alias for the group
         *
         * Create a request for the method "aliases.insert".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
         * Insert#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
         * be called to initialize this instance immediately after invoking the constructor. </p>
         *
         * @param groupKey Email or immutable Id of the group
         * @param content the {@link com.google.api.services.admin.directory.model.Alias}
         * @since 1.13
         */
        protected Insert(java.lang.String groupKey, com.google.api.services.admin.directory.model.Alias content) {
          super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.Alias.class);
          this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        }

        @Override
        public Insert setAlt(java.lang.String alt) {
          return (Insert) super.setAlt(alt);
        }

        @Override
        public Insert setFields(java.lang.String fields) {
          return (Insert) super.setFields(fields);
        }

        @Override
        public Insert setKey(java.lang.String key) {
          return (Insert) super.setKey(key);
        }

        @Override
        public Insert setOauthToken(java.lang.String oauthToken) {
          return (Insert) super.setOauthToken(oauthToken);
        }

        @Override
        public Insert setPrettyPrint(java.lang.Boolean prettyPrint) {
          return (Insert) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Insert setQuotaUser(java.lang.String quotaUser) {
          return (Insert) super.setQuotaUser(quotaUser);
        }

        @Override
        public Insert setUserIp(java.lang.String userIp) {
          return (Insert) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the group */
        @com.google.api.client.util.Key
        private java.lang.String groupKey;

        /** Email or immutable Id of the group
         */
        public java.lang.String getGroupKey() {
          return groupKey;
        }

        /** Email or immutable Id of the group */
        public Insert setGroupKey(java.lang.String groupKey) {
          this.groupKey = groupKey;
          return this;
        }

        @Override
        public Insert set(String parameterName, Object value) {
          return (Insert) super.set(parameterName, value);
        }
      }
      /**
       * List all aliases for a group
       *
       * Create a request for the method "aliases.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation.
       *
       * @param groupKey Email or immutable Id of the group
       * @return the request
       */
      public List list(java.lang.String groupKey) throws java.io.IOException {
        List result = new List(groupKey);
        initialize(result);
        return result;
      }

      public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.Aliases> {

        private static final String REST_PATH = "groups/{groupKey}/aliases";

        /**
         * List all aliases for a group
         *
         * Create a request for the method "aliases.list".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
         * List#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
         * called to initialize this instance immediately after invoking the constructor. </p>
         *
         * @param groupKey Email or immutable Id of the group
         * @since 1.13
         */
        protected List(java.lang.String groupKey) {
          super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Aliases.class);
          this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        }

        @Override
        public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
          return super.executeUsingHead();
        }

        @Override
        public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
          return super.buildHttpRequestUsingHead();
        }

        @Override
        public List setAlt(java.lang.String alt) {
          return (List) super.setAlt(alt);
        }

        @Override
        public List setFields(java.lang.String fields) {
          return (List) super.setFields(fields);
        }

        @Override
        public List setKey(java.lang.String key) {
          return (List) super.setKey(key);
        }

        @Override
        public List setOauthToken(java.lang.String oauthToken) {
          return (List) super.setOauthToken(oauthToken);
        }

        @Override
        public List setPrettyPrint(java.lang.Boolean prettyPrint) {
          return (List) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public List setQuotaUser(java.lang.String quotaUser) {
          return (List) super.setQuotaUser(quotaUser);
        }

        @Override
        public List setUserIp(java.lang.String userIp) {
          return (List) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the group */
        @com.google.api.client.util.Key
        private java.lang.String groupKey;

        /** Email or immutable Id of the group
         */
        public java.lang.String getGroupKey() {
          return groupKey;
        }

        /** Email or immutable Id of the group */
        public List setGroupKey(java.lang.String groupKey) {
          this.groupKey = groupKey;
          return this;
        }

        @Override
        public List set(String parameterName, Object value) {
          return (List) super.set(parameterName, value);
        }
      }

    }
  }

  /**
   * An accessor for creating requests from the Members collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Members.List request = admin.members().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Members members() {
    return new Members();
  }

  /**
   * The "members" collection of methods.
   */
  public class Members {

    /**
     * Remove membership.
     *
     * Create a request for the method "members.delete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @param memberKey Email or immutable Id of the member
     * @return the request
     */
    public Delete delete(java.lang.String groupKey, java.lang.String memberKey) throws java.io.IOException {
      Delete result = new Delete(groupKey, memberKey);
      initialize(result);
      return result;
    }

    public class Delete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "groups/{groupKey}/members/{memberKey}";

      /**
       * Remove membership.
       *
       * Create a request for the method "members.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @param memberKey Email or immutable Id of the member
       * @since 1.13
       */
      protected Delete(java.lang.String groupKey, java.lang.String memberKey) {
        super(Directory.this, "DELETE", REST_PATH, null, Void.class);
        this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        this.memberKey = com.google.api.client.util.Preconditions.checkNotNull(memberKey, "Required parameter memberKey must be specified.");
      }

      @Override
      public Delete setAlt(java.lang.String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(java.lang.String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(java.lang.String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(java.lang.String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(java.lang.String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(java.lang.String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private java.lang.String groupKey;

      /** Email or immutable Id of the group
       */
      public java.lang.String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public Delete setGroupKey(java.lang.String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      /** Email or immutable Id of the member */
      @com.google.api.client.util.Key
      private java.lang.String memberKey;

      /** Email or immutable Id of the member
       */
      public java.lang.String getMemberKey() {
        return memberKey;
      }

      /** Email or immutable Id of the member */
      public Delete setMemberKey(java.lang.String memberKey) {
        this.memberKey = memberKey;
        return this;
      }

      @Override
      public Delete set(String parameterName, Object value) {
        return (Delete) super.set(parameterName, value);
      }
    }
    /**
     * Retrieve Group Member
     *
     * Create a request for the method "members.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @param memberKey Email or immutable Id of the member
     * @return the request
     */
    public Get get(java.lang.String groupKey, java.lang.String memberKey) throws java.io.IOException {
      Get result = new Get(groupKey, memberKey);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.Member> {

      private static final String REST_PATH = "groups/{groupKey}/members/{memberKey}";

      /**
       * Retrieve Group Member
       *
       * Create a request for the method "members.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @param memberKey Email or immutable Id of the member
       * @since 1.13
       */
      protected Get(java.lang.String groupKey, java.lang.String memberKey) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Member.class);
        this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        this.memberKey = com.google.api.client.util.Preconditions.checkNotNull(memberKey, "Required parameter memberKey must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(java.lang.String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(java.lang.String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(java.lang.String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(java.lang.String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(java.lang.String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(java.lang.String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private java.lang.String groupKey;

      /** Email or immutable Id of the group
       */
      public java.lang.String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public Get setGroupKey(java.lang.String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      /** Email or immutable Id of the member */
      @com.google.api.client.util.Key
      private java.lang.String memberKey;

      /** Email or immutable Id of the member
       */
      public java.lang.String getMemberKey() {
        return memberKey;
      }

      /** Email or immutable Id of the member */
      public Get setMemberKey(java.lang.String memberKey) {
        this.memberKey = memberKey;
        return this;
      }

      @Override
      public Get set(String parameterName, Object value) {
        return (Get) super.set(parameterName, value);
      }
    }
    /**
     * Add user to the specified group.
     *
     * Create a request for the method "members.insert".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @param content the {@link com.google.api.services.admin.directory.model.Member}
     * @return the request
     */
    public Insert insert(java.lang.String groupKey, com.google.api.services.admin.directory.model.Member content) throws java.io.IOException {
      Insert result = new Insert(groupKey, content);
      initialize(result);
      return result;
    }

    public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.Member> {

      private static final String REST_PATH = "groups/{groupKey}/members";

      /**
       * Add user to the specified group.
       *
       * Create a request for the method "members.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @param content the {@link com.google.api.services.admin.directory.model.Member}
       * @since 1.13
       */
      protected Insert(java.lang.String groupKey, com.google.api.services.admin.directory.model.Member content) {
        super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.Member.class);
        this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public Insert setAlt(java.lang.String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(java.lang.String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(java.lang.String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(java.lang.String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(java.lang.String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(java.lang.String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private java.lang.String groupKey;

      /** Email or immutable Id of the group
       */
      public java.lang.String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public Insert setGroupKey(java.lang.String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      @Override
      public Insert set(String parameterName, Object value) {
        return (Insert) super.set(parameterName, value);
      }
    }
    /**
     * Retrieve all members in a group (paginated)
     *
     * Create a request for the method "members.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group
     * @return the request
     */
    public List list(java.lang.String groupKey) throws java.io.IOException {
      List result = new List(groupKey);
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.Members> {

      private static final String REST_PATH = "groups/{groupKey}/members";

      /**
       * Retrieve all members in a group (paginated)
       *
       * Create a request for the method "members.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group
       * @since 1.13
       */
      protected List(java.lang.String groupKey) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Members.class);
        this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(java.lang.String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(java.lang.String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(java.lang.String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(java.lang.String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(java.lang.String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(java.lang.String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group */
      @com.google.api.client.util.Key
      private java.lang.String groupKey;

      /** Email or immutable Id of the group
       */
      public java.lang.String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group */
      public List setGroupKey(java.lang.String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      /** Token to specify next page in the list */
      @com.google.api.client.util.Key
      private java.lang.String pageToken;

      /** Token to specify next page in the list
       */
      public java.lang.String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page in the list */
      public List setPageToken(java.lang.String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /** Comma separated role values to filter list results on. */
      @com.google.api.client.util.Key
      private java.lang.String roles;

      /** Comma separated role values to filter list results on.
       */
      public java.lang.String getRoles() {
        return roles;
      }

      /** Comma separated role values to filter list results on. */
      public List setRoles(java.lang.String roles) {
        this.roles = roles;
        return this;
      }

      /** Maximum number of results to return. Default is 200 */
      @com.google.api.client.util.Key
      private java.lang.Integer maxResults;

      /** Maximum number of results to return. Default is 200

     [minimum: 1]
       */
      public java.lang.Integer getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. Default is 200 */
      public List setMaxResults(java.lang.Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      @Override
      public List set(String parameterName, Object value) {
        return (List) super.set(parameterName, value);
      }
    }
    /**
     * Update membership of a user in the specified group. This method supports patch semantics.
     *
     * Create a request for the method "members.patch".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
     * @param memberKey Email or immutable Id of the user. If Id, it should match with id of member object
     * @param content the {@link com.google.api.services.admin.directory.model.Member}
     * @return the request
     */
    public Patch patch(java.lang.String groupKey, java.lang.String memberKey, com.google.api.services.admin.directory.model.Member content) throws java.io.IOException {
      Patch result = new Patch(groupKey, memberKey, content);
      initialize(result);
      return result;
    }

    public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.Member> {

      private static final String REST_PATH = "groups/{groupKey}/members/{memberKey}";

      /**
       * Update membership of a user in the specified group. This method supports patch semantics.
       *
       * Create a request for the method "members.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
       * @param memberKey Email or immutable Id of the user. If Id, it should match with id of member object
       * @param content the {@link com.google.api.services.admin.directory.model.Member}
       * @since 1.13
       */
      protected Patch(java.lang.String groupKey, java.lang.String memberKey, com.google.api.services.admin.directory.model.Member content) {
        super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.Member.class);
        this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        this.memberKey = com.google.api.client.util.Preconditions.checkNotNull(memberKey, "Required parameter memberKey must be specified.");
      }

      @Override
      public Patch setAlt(java.lang.String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(java.lang.String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(java.lang.String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(java.lang.String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(java.lang.String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(java.lang.String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      @com.google.api.client.util.Key
      private java.lang.String groupKey;

      /** Email or immutable Id of the group. If Id, it should match with id of group object
       */
      public java.lang.String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      public Patch setGroupKey(java.lang.String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of member object */
      @com.google.api.client.util.Key
      private java.lang.String memberKey;

      /** Email or immutable Id of the user. If Id, it should match with id of member object
       */
      public java.lang.String getMemberKey() {
        return memberKey;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of member object */
      public Patch setMemberKey(java.lang.String memberKey) {
        this.memberKey = memberKey;
        return this;
      }

      @Override
      public Patch set(String parameterName, Object value) {
        return (Patch) super.set(parameterName, value);
      }
    }
    /**
     * Update membership of a user in the specified group.
     *
     * Create a request for the method "members.update".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
     * @param memberKey Email or immutable Id of the user. If Id, it should match with id of member object
     * @param content the {@link com.google.api.services.admin.directory.model.Member}
     * @return the request
     */
    public Update update(java.lang.String groupKey, java.lang.String memberKey, com.google.api.services.admin.directory.model.Member content) throws java.io.IOException {
      Update result = new Update(groupKey, memberKey, content);
      initialize(result);
      return result;
    }

    public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.Member> {

      private static final String REST_PATH = "groups/{groupKey}/members/{memberKey}";

      /**
       * Update membership of a user in the specified group.
       *
       * Create a request for the method "members.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param groupKey Email or immutable Id of the group. If Id, it should match with id of group object
       * @param memberKey Email or immutable Id of the user. If Id, it should match with id of member object
       * @param content the {@link com.google.api.services.admin.directory.model.Member}
       * @since 1.13
       */
      protected Update(java.lang.String groupKey, java.lang.String memberKey, com.google.api.services.admin.directory.model.Member content) {
        super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.Member.class);
        this.groupKey = com.google.api.client.util.Preconditions.checkNotNull(groupKey, "Required parameter groupKey must be specified.");
        this.memberKey = com.google.api.client.util.Preconditions.checkNotNull(memberKey, "Required parameter memberKey must be specified.");
      }

      @Override
      public Update setAlt(java.lang.String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(java.lang.String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(java.lang.String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(java.lang.String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(java.lang.String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(java.lang.String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      @com.google.api.client.util.Key
      private java.lang.String groupKey;

      /** Email or immutable Id of the group. If Id, it should match with id of group object
       */
      public java.lang.String getGroupKey() {
        return groupKey;
      }

      /** Email or immutable Id of the group. If Id, it should match with id of group object */
      public Update setGroupKey(java.lang.String groupKey) {
        this.groupKey = groupKey;
        return this;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of member object */
      @com.google.api.client.util.Key
      private java.lang.String memberKey;

      /** Email or immutable Id of the user. If Id, it should match with id of member object
       */
      public java.lang.String getMemberKey() {
        return memberKey;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of member object */
      public Update setMemberKey(java.lang.String memberKey) {
        this.memberKey = memberKey;
        return this;
      }

      @Override
      public Update set(String parameterName, Object value) {
        return (Update) super.set(parameterName, value);
      }
    }

  }

  /**
   * An accessor for creating requests from the Mobiledevices collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Mobiledevices.List request = admin.mobiledevices().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Mobiledevices mobiledevices() {
    return new Mobiledevices();
  }

  /**
   * The "mobiledevices" collection of methods.
   */
  public class Mobiledevices {

    /**
     * Take action on Mobile Device
     *
     * Create a request for the method "mobiledevices.action".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Action#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param resourceId Immutable id of Mobile Device
     * @param content the {@link com.google.api.services.admin.directory.model.MobileDeviceAction}
     * @return the request
     */
    public Action action(java.lang.String customerId, java.lang.String resourceId, com.google.api.services.admin.directory.model.MobileDeviceAction content) throws java.io.IOException {
      Action result = new Action(customerId, resourceId, content);
      initialize(result);
      return result;
    }

    public class Action extends DirectoryRequest<Void> {

      private static final String REST_PATH = "customer/{customerId}/devices/mobile/{resourceId}/action";

      /**
       * Take action on Mobile Device
       *
       * Create a request for the method "mobiledevices.action".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Action#execute()} method to invoke the remote operation. <p> {@link
       * Action#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param resourceId Immutable id of Mobile Device
       * @param content the {@link com.google.api.services.admin.directory.model.MobileDeviceAction}
       * @since 1.13
       */
      protected Action(java.lang.String customerId, java.lang.String resourceId, com.google.api.services.admin.directory.model.MobileDeviceAction content) {
        super(Directory.this, "POST", REST_PATH, content, Void.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.resourceId = com.google.api.client.util.Preconditions.checkNotNull(resourceId, "Required parameter resourceId must be specified.");
      }

      @Override
      public Action setAlt(java.lang.String alt) {
        return (Action) super.setAlt(alt);
      }

      @Override
      public Action setFields(java.lang.String fields) {
        return (Action) super.setFields(fields);
      }

      @Override
      public Action setKey(java.lang.String key) {
        return (Action) super.setKey(key);
      }

      @Override
      public Action setOauthToken(java.lang.String oauthToken) {
        return (Action) super.setOauthToken(oauthToken);
      }

      @Override
      public Action setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Action) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Action setQuotaUser(java.lang.String quotaUser) {
        return (Action) super.setQuotaUser(quotaUser);
      }

      @Override
      public Action setUserIp(java.lang.String userIp) {
        return (Action) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Action setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Mobile Device */
      @com.google.api.client.util.Key
      private java.lang.String resourceId;

      /** Immutable id of Mobile Device
       */
      public java.lang.String getResourceId() {
        return resourceId;
      }

      /** Immutable id of Mobile Device */
      public Action setResourceId(java.lang.String resourceId) {
        this.resourceId = resourceId;
        return this;
      }

      @Override
      public Action set(String parameterName, Object value) {
        return (Action) super.set(parameterName, value);
      }
    }
    /**
     * Delete Mobile Device
     *
     * Create a request for the method "mobiledevices.delete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param resourceId Immutable id of Mobile Device
     * @return the request
     */
    public Delete delete(java.lang.String customerId, java.lang.String resourceId) throws java.io.IOException {
      Delete result = new Delete(customerId, resourceId);
      initialize(result);
      return result;
    }

    public class Delete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "customer/{customerId}/devices/mobile/{resourceId}";

      /**
       * Delete Mobile Device
       *
       * Create a request for the method "mobiledevices.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param resourceId Immutable id of Mobile Device
       * @since 1.13
       */
      protected Delete(java.lang.String customerId, java.lang.String resourceId) {
        super(Directory.this, "DELETE", REST_PATH, null, Void.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.resourceId = com.google.api.client.util.Preconditions.checkNotNull(resourceId, "Required parameter resourceId must be specified.");
      }

      @Override
      public Delete setAlt(java.lang.String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(java.lang.String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(java.lang.String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(java.lang.String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(java.lang.String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(java.lang.String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Delete setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Mobile Device */
      @com.google.api.client.util.Key
      private java.lang.String resourceId;

      /** Immutable id of Mobile Device
       */
      public java.lang.String getResourceId() {
        return resourceId;
      }

      /** Immutable id of Mobile Device */
      public Delete setResourceId(java.lang.String resourceId) {
        this.resourceId = resourceId;
        return this;
      }

      @Override
      public Delete set(String parameterName, Object value) {
        return (Delete) super.set(parameterName, value);
      }
    }
    /**
     * Retrieve Mobile Device
     *
     * Create a request for the method "mobiledevices.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param resourceId Immutable id of Mobile Device
     * @return the request
     */
    public Get get(java.lang.String customerId, java.lang.String resourceId) throws java.io.IOException {
      Get result = new Get(customerId, resourceId);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.MobileDevice> {

      private static final String REST_PATH = "customer/{customerId}/devices/mobile/{resourceId}";

      /**
       * Retrieve Mobile Device
       *
       * Create a request for the method "mobiledevices.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param resourceId Immutable id of Mobile Device
       * @since 1.13
       */
      protected Get(java.lang.String customerId, java.lang.String resourceId) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.MobileDevice.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.resourceId = com.google.api.client.util.Preconditions.checkNotNull(resourceId, "Required parameter resourceId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(java.lang.String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(java.lang.String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(java.lang.String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(java.lang.String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(java.lang.String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(java.lang.String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Get setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Immutable id of Mobile Device */
      @com.google.api.client.util.Key
      private java.lang.String resourceId;

      /** Immutable id of Mobile Device
       */
      public java.lang.String getResourceId() {
        return resourceId;
      }

      /** Immutable id of Mobile Device */
      public Get setResourceId(java.lang.String resourceId) {
        this.resourceId = resourceId;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private java.lang.String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public java.lang.String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public Get setProjection(java.lang.String projection) {
        this.projection = projection;
        return this;
      }

      @Override
      public Get set(String parameterName, Object value) {
        return (Get) super.set(parameterName, value);
      }
    }
    /**
     * Retrieve all Mobile Devices of a customer (paginated)
     *
     * Create a request for the method "mobiledevices.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @return the request
     */
    public List list(java.lang.String customerId) throws java.io.IOException {
      List result = new List(customerId);
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.MobileDevices> {

      private static final String REST_PATH = "customer/{customerId}/devices/mobile";

      /**
       * Retrieve all Mobile Devices of a customer (paginated)
       *
       * Create a request for the method "mobiledevices.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @since 1.13
       */
      protected List(java.lang.String customerId) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.MobileDevices.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(java.lang.String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(java.lang.String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(java.lang.String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(java.lang.String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(java.lang.String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(java.lang.String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public List setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Column to use for sorting results */
      @com.google.api.client.util.Key
      private java.lang.String orderBy;

      /** Column to use for sorting results
       */
      public java.lang.String getOrderBy() {
        return orderBy;
      }

      /** Column to use for sorting results */
      public List setOrderBy(java.lang.String orderBy) {
        this.orderBy = orderBy;
        return this;
      }

      /** Restrict information returned to a set of selected fields. */
      @com.google.api.client.util.Key
      private java.lang.String projection;

      /** Restrict information returned to a set of selected fields.
       */
      public java.lang.String getProjection() {
        return projection;
      }

      /** Restrict information returned to a set of selected fields. */
      public List setProjection(java.lang.String projection) {
        this.projection = projection;
        return this;
      }

      /** Maximum number of results to return. Default is 100 */
      @com.google.api.client.util.Key
      private java.lang.Integer maxResults;

      /** Maximum number of results to return. Default is 100

     [minimum: 1]
       */
      public java.lang.Integer getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. Default is 100 */
      public List setMaxResults(java.lang.Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** Token to specify next page in the list */
      @com.google.api.client.util.Key
      private java.lang.String pageToken;

      /** Token to specify next page in the list
       */
      public java.lang.String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page in the list */
      public List setPageToken(java.lang.String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**
       * Whether to return results in ascending or descending order. Only of use when orderBy is
       * also used
       */
      @com.google.api.client.util.Key
      private java.lang.String sortOrder;

      /** Whether to return results in ascending or descending order. Only of use when orderBy is also used
       */
      public java.lang.String getSortOrder() {
        return sortOrder;
      }

      /**
       * Whether to return results in ascending or descending order. Only of use when orderBy is
       * also used
       */
      public List setSortOrder(java.lang.String sortOrder) {
        this.sortOrder = sortOrder;
        return this;
      }

      /**
       * Search string in the format given at
       * http://support.google.com/a/bin/answer.py?hl=en=1408863#search
       */
      @com.google.api.client.util.Key
      private java.lang.String query;

      /** Search string in the format given at http://support.google.com/a/bin/answer.py?hl=en=1408863#search
       */
      public java.lang.String getQuery() {
        return query;
      }

      /**
       * Search string in the format given at
       * http://support.google.com/a/bin/answer.py?hl=en=1408863#search
       */
      public List setQuery(java.lang.String query) {
        this.query = query;
        return this;
      }

      @Override
      public List set(String parameterName, Object value) {
        return (List) super.set(parameterName, value);
      }
    }

  }

  /**
   * An accessor for creating requests from the Orgunits collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Orgunits.List request = admin.orgunits().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Orgunits orgunits() {
    return new Orgunits();
  }

  /**
   * The "orgunits" collection of methods.
   */
  public class Orgunits {

    /**
     * Remove Organization Unit
     *
     * Create a request for the method "orgunits.delete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param orgUnitPath Full path of the organization unit
     * @return the request
     */
    public Delete delete(java.lang.String customerId, java.util.List<java.lang.String> orgUnitPath) throws java.io.IOException {
      Delete result = new Delete(customerId, orgUnitPath);
      initialize(result);
      return result;
    }

    public class Delete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "customer/{customerId}/orgunits{/orgUnitPath*}";

      /**
       * Remove Organization Unit
       *
       * Create a request for the method "orgunits.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param orgUnitPath Full path of the organization unit
       * @since 1.13
       */
      protected Delete(java.lang.String customerId, java.util.List<java.lang.String> orgUnitPath) {
        super(Directory.this, "DELETE", REST_PATH, null, Void.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.orgUnitPath = com.google.api.client.util.Preconditions.checkNotNull(orgUnitPath, "Required parameter orgUnitPath must be specified.");
      }

      @Override
      public Delete setAlt(java.lang.String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(java.lang.String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(java.lang.String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(java.lang.String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(java.lang.String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(java.lang.String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Delete setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Full path of the organization unit */
      @com.google.api.client.util.Key
      private java.util.List<java.lang.String> orgUnitPath;

      /** Full path of the organization unit
       */
      public java.util.List<java.lang.String> getOrgUnitPath() {
        return orgUnitPath;
      }

      /** Full path of the organization unit */
      public Delete setOrgUnitPath(java.util.List<java.lang.String> orgUnitPath) {
        this.orgUnitPath = orgUnitPath;
        return this;
      }

      @Override
      public Delete set(String parameterName, Object value) {
        return (Delete) super.set(parameterName, value);
      }
    }
    /**
     * Retrieve Organization Unit
     *
     * Create a request for the method "orgunits.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param orgUnitPath Full path of the organization unit
     * @return the request
     */
    public Get get(java.lang.String customerId, java.util.List<java.lang.String> orgUnitPath) throws java.io.IOException {
      Get result = new Get(customerId, orgUnitPath);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.OrgUnit> {

      private static final String REST_PATH = "customer/{customerId}/orgunits{/orgUnitPath*}";

      /**
       * Retrieve Organization Unit
       *
       * Create a request for the method "orgunits.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param orgUnitPath Full path of the organization unit
       * @since 1.13
       */
      protected Get(java.lang.String customerId, java.util.List<java.lang.String> orgUnitPath) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.OrgUnit.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.orgUnitPath = com.google.api.client.util.Preconditions.checkNotNull(orgUnitPath, "Required parameter orgUnitPath must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(java.lang.String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(java.lang.String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(java.lang.String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(java.lang.String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(java.lang.String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(java.lang.String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Get setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Full path of the organization unit */
      @com.google.api.client.util.Key
      private java.util.List<java.lang.String> orgUnitPath;

      /** Full path of the organization unit
       */
      public java.util.List<java.lang.String> getOrgUnitPath() {
        return orgUnitPath;
      }

      /** Full path of the organization unit */
      public Get setOrgUnitPath(java.util.List<java.lang.String> orgUnitPath) {
        this.orgUnitPath = orgUnitPath;
        return this;
      }

      @Override
      public Get set(String parameterName, Object value) {
        return (Get) super.set(parameterName, value);
      }
    }
    /**
     * Add Organization Unit
     *
     * Create a request for the method "orgunits.insert".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
     * @return the request
     */
    public Insert insert(java.lang.String customerId, com.google.api.services.admin.directory.model.OrgUnit content) throws java.io.IOException {
      Insert result = new Insert(customerId, content);
      initialize(result);
      return result;
    }

    public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.OrgUnit> {

      private static final String REST_PATH = "customer/{customerId}/orgunits";

      /**
       * Add Organization Unit
       *
       * Create a request for the method "orgunits.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
       * @since 1.13
       */
      protected Insert(java.lang.String customerId, com.google.api.services.admin.directory.model.OrgUnit content) {
        super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.OrgUnit.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        checkRequiredParameter(content, "content");
        checkRequiredParameter(content.getName(), "OrgUnit.getName()");
        checkRequiredParameter(content, "content");
        checkRequiredParameter(content.getParentOrgUnitPath(), "OrgUnit.getParentOrgUnitPath()");
      }

      @Override
      public Insert setAlt(java.lang.String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(java.lang.String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(java.lang.String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(java.lang.String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(java.lang.String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(java.lang.String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Insert setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      @Override
      public Insert set(String parameterName, Object value) {
        return (Insert) super.set(parameterName, value);
      }
    }
    /**
     * Retrieve all Organization Units
     *
     * Create a request for the method "orgunits.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @return the request
     */
    public List list(java.lang.String customerId) throws java.io.IOException {
      List result = new List(customerId);
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.OrgUnits> {

      private static final String REST_PATH = "customer/{customerId}/orgunits";

      /**
       * Retrieve all Organization Units
       *
       * Create a request for the method "orgunits.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @since 1.13
       */
      protected List(java.lang.String customerId) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.OrgUnits.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(java.lang.String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(java.lang.String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(java.lang.String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(java.lang.String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(java.lang.String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(java.lang.String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public List setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Whether to return all sub-organizations or just immediate children */
      @com.google.api.client.util.Key
      private java.lang.String type;

      /** Whether to return all sub-organizations or just immediate children
       */
      public java.lang.String getType() {
        return type;
      }

      /** Whether to return all sub-organizations or just immediate children */
      public List setType(java.lang.String type) {
        this.type = type;
        return this;
      }

      /** the URL-encoded organization unit */
      @com.google.api.client.util.Key
      private java.lang.String orgUnitPath;

      /** the URL-encoded organization unit
       */
      public java.lang.String getOrgUnitPath() {
        return orgUnitPath;
      }

      /** the URL-encoded organization unit */
      public List setOrgUnitPath(java.lang.String orgUnitPath) {
        this.orgUnitPath = orgUnitPath;
        return this;
      }

      @Override
      public List set(String parameterName, Object value) {
        return (List) super.set(parameterName, value);
      }
    }
    /**
     * Update Organization Unit. This method supports patch semantics.
     *
     * Create a request for the method "orgunits.patch".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param orgUnitPath Full path of the organization unit
     * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
     * @return the request
     */
    public Patch patch(java.lang.String customerId, java.util.List<java.lang.String> orgUnitPath, com.google.api.services.admin.directory.model.OrgUnit content) throws java.io.IOException {
      Patch result = new Patch(customerId, orgUnitPath, content);
      initialize(result);
      return result;
    }

    public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.OrgUnit> {

      private static final String REST_PATH = "customer/{customerId}/orgunits{/orgUnitPath*}";

      /**
       * Update Organization Unit. This method supports patch semantics.
       *
       * Create a request for the method "orgunits.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param orgUnitPath Full path of the organization unit
       * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
       * @since 1.13
       */
      protected Patch(java.lang.String customerId, java.util.List<java.lang.String> orgUnitPath, com.google.api.services.admin.directory.model.OrgUnit content) {
        super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.OrgUnit.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.orgUnitPath = com.google.api.client.util.Preconditions.checkNotNull(orgUnitPath, "Required parameter orgUnitPath must be specified.");
      }

      @Override
      public Patch setAlt(java.lang.String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(java.lang.String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(java.lang.String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(java.lang.String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(java.lang.String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(java.lang.String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Patch setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Full path of the organization unit */
      @com.google.api.client.util.Key
      private java.util.List<java.lang.String> orgUnitPath;

      /** Full path of the organization unit
       */
      public java.util.List<java.lang.String> getOrgUnitPath() {
        return orgUnitPath;
      }

      /** Full path of the organization unit */
      public Patch setOrgUnitPath(java.util.List<java.lang.String> orgUnitPath) {
        this.orgUnitPath = orgUnitPath;
        return this;
      }

      @Override
      public Patch set(String parameterName, Object value) {
        return (Patch) super.set(parameterName, value);
      }
    }
    /**
     * Update Organization Unit
     *
     * Create a request for the method "orgunits.update".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param customerId Immutable id of the Google Apps account
     * @param orgUnitPath Full path of the organization unit
     * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
     * @return the request
     */
    public Update update(java.lang.String customerId, java.util.List<java.lang.String> orgUnitPath, com.google.api.services.admin.directory.model.OrgUnit content) throws java.io.IOException {
      Update result = new Update(customerId, orgUnitPath, content);
      initialize(result);
      return result;
    }

    public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.OrgUnit> {

      private static final String REST_PATH = "customer/{customerId}/orgunits{/orgUnitPath*}";

      /**
       * Update Organization Unit
       *
       * Create a request for the method "orgunits.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param customerId Immutable id of the Google Apps account
       * @param orgUnitPath Full path of the organization unit
       * @param content the {@link com.google.api.services.admin.directory.model.OrgUnit}
       * @since 1.13
       */
      protected Update(java.lang.String customerId, java.util.List<java.lang.String> orgUnitPath, com.google.api.services.admin.directory.model.OrgUnit content) {
        super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.OrgUnit.class);
        this.customerId = com.google.api.client.util.Preconditions.checkNotNull(customerId, "Required parameter customerId must be specified.");
        this.orgUnitPath = com.google.api.client.util.Preconditions.checkNotNull(orgUnitPath, "Required parameter orgUnitPath must be specified.");
      }

      @Override
      public Update setAlt(java.lang.String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(java.lang.String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(java.lang.String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(java.lang.String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(java.lang.String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(java.lang.String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Immutable id of the Google Apps account */
      @com.google.api.client.util.Key
      private java.lang.String customerId;

      /** Immutable id of the Google Apps account
       */
      public java.lang.String getCustomerId() {
        return customerId;
      }

      /** Immutable id of the Google Apps account */
      public Update setCustomerId(java.lang.String customerId) {
        this.customerId = customerId;
        return this;
      }

      /** Full path of the organization unit */
      @com.google.api.client.util.Key
      private java.util.List<java.lang.String> orgUnitPath;

      /** Full path of the organization unit
       */
      public java.util.List<java.lang.String> getOrgUnitPath() {
        return orgUnitPath;
      }

      /** Full path of the organization unit */
      public Update setOrgUnitPath(java.util.List<java.lang.String> orgUnitPath) {
        this.orgUnitPath = orgUnitPath;
        return this;
      }

      @Override
      public Update set(String parameterName, Object value) {
        return (Update) super.set(parameterName, value);
      }
    }

  }

  /**
   * An accessor for creating requests from the Users collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Directory admin = new Directory(...);}
   *   {@code Directory.Users.List request = admin.users().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Users users() {
    return new Users();
  }

  /**
   * The "users" collection of methods.
   */
  public class Users {

    /**
     * Delete user
     *
     * Create a request for the method "users.delete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
     *
     * @param userKey Email or immutable Id of the user
     * @return the request
     */
    public Delete delete(java.lang.String userKey) throws java.io.IOException {
      Delete result = new Delete(userKey);
      initialize(result);
      return result;
    }

    public class Delete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "users/{userKey}";

      /**
       * Delete user
       *
       * Create a request for the method "users.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
       * Delete#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param userKey Email or immutable Id of the user
       * @since 1.13
       */
      protected Delete(java.lang.String userKey) {
        super(Directory.this, "DELETE", REST_PATH, null, Void.class);
        this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public Delete setAlt(java.lang.String alt) {
        return (Delete) super.setAlt(alt);
      }

      @Override
      public Delete setFields(java.lang.String fields) {
        return (Delete) super.setFields(fields);
      }

      @Override
      public Delete setKey(java.lang.String key) {
        return (Delete) super.setKey(key);
      }

      @Override
      public Delete setOauthToken(java.lang.String oauthToken) {
        return (Delete) super.setOauthToken(oauthToken);
      }

      @Override
      public Delete setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Delete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Delete setQuotaUser(java.lang.String quotaUser) {
        return (Delete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Delete setUserIp(java.lang.String userIp) {
        return (Delete) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the user */
      @com.google.api.client.util.Key
      private java.lang.String userKey;

      /** Email or immutable Id of the user
       */
      public java.lang.String getUserKey() {
        return userKey;
      }

      /** Email or immutable Id of the user */
      public Delete setUserKey(java.lang.String userKey) {
        this.userKey = userKey;
        return this;
      }

      @Override
      public Delete set(String parameterName, Object value) {
        return (Delete) super.set(parameterName, value);
      }
    }
    /**
     * retrieve user
     *
     * Create a request for the method "users.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param userKey Email or immutable Id of the user
     * @return the request
     */
    public Get get(java.lang.String userKey) throws java.io.IOException {
      Get result = new Get(userKey);
      initialize(result);
      return result;
    }

    public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.User> {

      private static final String REST_PATH = "users/{userKey}";

      /**
       * retrieve user
       *
       * Create a request for the method "users.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param userKey Email or immutable Id of the user
       * @since 1.13
       */
      protected Get(java.lang.String userKey) {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.User.class);
        this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(java.lang.String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(java.lang.String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(java.lang.String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(java.lang.String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(java.lang.String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(java.lang.String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the user */
      @com.google.api.client.util.Key
      private java.lang.String userKey;

      /** Email or immutable Id of the user
       */
      public java.lang.String getUserKey() {
        return userKey;
      }

      /** Email or immutable Id of the user */
      public Get setUserKey(java.lang.String userKey) {
        this.userKey = userKey;
        return this;
      }

      @Override
      public Get set(String parameterName, Object value) {
        return (Get) super.set(parameterName, value);
      }
    }
    /**
     * create user.
     *
     * Create a request for the method "users.insert".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
     *
     * @param content the {@link com.google.api.services.admin.directory.model.User}
     * @return the request
     */
    public Insert insert(com.google.api.services.admin.directory.model.User content) throws java.io.IOException {
      Insert result = new Insert(content);
      initialize(result);
      return result;
    }

    public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.User> {

      private static final String REST_PATH = "users";

      /**
       * create user.
       *
       * Create a request for the method "users.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
       * Insert#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param content the {@link com.google.api.services.admin.directory.model.User}
       * @since 1.13
       */
      protected Insert(com.google.api.services.admin.directory.model.User content) {
        super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.User.class);
        checkRequiredParameter(content, "content");
        checkRequiredParameter(content.getPassword(), "User.getPassword()");
        checkRequiredParameter(content, "content");
        checkRequiredParameter(content.getPrimaryEmail(), "User.getPrimaryEmail()");
      }

      @Override
      public Insert setAlt(java.lang.String alt) {
        return (Insert) super.setAlt(alt);
      }

      @Override
      public Insert setFields(java.lang.String fields) {
        return (Insert) super.setFields(fields);
      }

      @Override
      public Insert setKey(java.lang.String key) {
        return (Insert) super.setKey(key);
      }

      @Override
      public Insert setOauthToken(java.lang.String oauthToken) {
        return (Insert) super.setOauthToken(oauthToken);
      }

      @Override
      public Insert setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Insert) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Insert setQuotaUser(java.lang.String quotaUser) {
        return (Insert) super.setQuotaUser(quotaUser);
      }

      @Override
      public Insert setUserIp(java.lang.String userIp) {
        return (Insert) super.setUserIp(userIp);
      }

      @Override
      public Insert set(String parameterName, Object value) {
        return (Insert) super.set(parameterName, value);
      }
    }
    /**
     * Retrieve either deleted users or all users in a domain (paginated)
     *
     * Create a request for the method "users.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @return the request
     */
    public List list() throws java.io.IOException {
      List result = new List();
      initialize(result);
      return result;
    }

    public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.Users> {

      private static final String REST_PATH = "users";

      /**
       * Retrieve either deleted users or all users in a domain (paginated)
       *
       * Create a request for the method "users.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
       * called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @since 1.13
       */
      protected List() {
        super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Users.class);
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(java.lang.String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(java.lang.String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(java.lang.String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(java.lang.String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(java.lang.String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(java.lang.String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /**
       * Immutable id of the Google Apps account. In case of multi-domain, to fetch all users for a
       * customer, fill this field instead of domain.
       */
      @com.google.api.client.util.Key
      private java.lang.String customer;

      /** Immutable id of the Google Apps account. In case of multi-domain, to fetch all users for a
     customer, fill this field instead of domain.
       */
      public java.lang.String getCustomer() {
        return customer;
      }

      /**
       * Immutable id of the Google Apps account. In case of multi-domain, to fetch all users for a
       * customer, fill this field instead of domain.
       */
      public List setCustomer(java.lang.String customer) {
        this.customer = customer;
        return this;
      }

      /** Column to use for sorting results */
      @com.google.api.client.util.Key
      private java.lang.String orderBy;

      /** Column to use for sorting results
       */
      public java.lang.String getOrderBy() {
        return orderBy;
      }

      /** Column to use for sorting results */
      public List setOrderBy(java.lang.String orderBy) {
        this.orderBy = orderBy;
        return this;
      }

      /**
       * Name of the domain. Fill this field to get users from only this domain. To return all users
       * in a multi-domain fill customer field instead.
       */
      @com.google.api.client.util.Key
      private java.lang.String domain;

      /** Name of the domain. Fill this field to get users from only this domain. To return all users in a
     multi-domain fill customer field instead.
       */
      public java.lang.String getDomain() {
        return domain;
      }

      /**
       * Name of the domain. Fill this field to get users from only this domain. To return all users
       * in a multi-domain fill customer field instead.
       */
      public List setDomain(java.lang.String domain) {
        this.domain = domain;
        return this;
      }

      /** If set to true retrieves the list of deleted users. Default is false */
      @com.google.api.client.util.Key
      private java.lang.String showDeleted;

      /** If set to true retrieves the list of deleted users. Default is false
       */
      public java.lang.String getShowDeleted() {
        return showDeleted;
      }

      /** If set to true retrieves the list of deleted users. Default is false */
      public List setShowDeleted(java.lang.String showDeleted) {
        this.showDeleted = showDeleted;
        return this;
      }

      /** Maximum number of results to return. Default is 100. Max allowed is 500 */
      @com.google.api.client.util.Key
      private java.lang.Integer maxResults;

      /** Maximum number of results to return. Default is 100. Max allowed is 500

     [minimum: 1] [maximum: 500]
       */
      public java.lang.Integer getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. Default is 100. Max allowed is 500 */
      public List setMaxResults(java.lang.Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** Token to specify next page in the list */
      @com.google.api.client.util.Key
      private java.lang.String pageToken;

      /** Token to specify next page in the list
       */
      public java.lang.String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page in the list */
      public List setPageToken(java.lang.String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /** Whether to return results in ascending or descending order. */
      @com.google.api.client.util.Key
      private java.lang.String sortOrder;

      /** Whether to return results in ascending or descending order.
       */
      public java.lang.String getSortOrder() {
        return sortOrder;
      }

      /** Whether to return results in ascending or descending order. */
      public List setSortOrder(java.lang.String sortOrder) {
        this.sortOrder = sortOrder;
        return this;
      }

      /**
       * Query string for prefix matching searches. Should be of the form "key:value*" where key can
       * be "email", "givenName" or "familyName". The asterisk is required, for example:
       * "givenName:Ann*" is a valid query.
       */
      @com.google.api.client.util.Key
      private java.lang.String query;

      /** Query string for prefix matching searches. Should be of the form "key:value*" where key can be
     "email", "givenName" or "familyName". The asterisk is required, for example: "givenName:Ann*" is a
     valid query.
       */
      public java.lang.String getQuery() {
        return query;
      }

      /**
       * Query string for prefix matching searches. Should be of the form "key:value*" where key can
       * be "email", "givenName" or "familyName". The asterisk is required, for example:
       * "givenName:Ann*" is a valid query.
       */
      public List setQuery(java.lang.String query) {
        this.query = query;
        return this;
      }

      @Override
      public List set(String parameterName, Object value) {
        return (List) super.set(parameterName, value);
      }
    }
    /**
     * change admin status of a user
     *
     * Create a request for the method "users.makeAdmin".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link MakeAdmin#execute()} method to invoke the remote operation.
     *
     * @param userKey Email or immutable Id of the user as admin
     * @param content the {@link com.google.api.services.admin.directory.model.UserMakeAdmin}
     * @return the request
     */
    public MakeAdmin makeAdmin(java.lang.String userKey, com.google.api.services.admin.directory.model.UserMakeAdmin content) throws java.io.IOException {
      MakeAdmin result = new MakeAdmin(userKey, content);
      initialize(result);
      return result;
    }

    public class MakeAdmin extends DirectoryRequest<Void> {

      private static final String REST_PATH = "users/{userKey}/makeAdmin";

      /**
       * change admin status of a user
       *
       * Create a request for the method "users.makeAdmin".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link MakeAdmin#execute()} method to invoke the remote operation. <p>
       * {@link
       * MakeAdmin#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)}
       * must be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param userKey Email or immutable Id of the user as admin
       * @param content the {@link com.google.api.services.admin.directory.model.UserMakeAdmin}
       * @since 1.13
       */
      protected MakeAdmin(java.lang.String userKey, com.google.api.services.admin.directory.model.UserMakeAdmin content) {
        super(Directory.this, "POST", REST_PATH, content, Void.class);
        this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public MakeAdmin setAlt(java.lang.String alt) {
        return (MakeAdmin) super.setAlt(alt);
      }

      @Override
      public MakeAdmin setFields(java.lang.String fields) {
        return (MakeAdmin) super.setFields(fields);
      }

      @Override
      public MakeAdmin setKey(java.lang.String key) {
        return (MakeAdmin) super.setKey(key);
      }

      @Override
      public MakeAdmin setOauthToken(java.lang.String oauthToken) {
        return (MakeAdmin) super.setOauthToken(oauthToken);
      }

      @Override
      public MakeAdmin setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (MakeAdmin) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public MakeAdmin setQuotaUser(java.lang.String quotaUser) {
        return (MakeAdmin) super.setQuotaUser(quotaUser);
      }

      @Override
      public MakeAdmin setUserIp(java.lang.String userIp) {
        return (MakeAdmin) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the user as admin */
      @com.google.api.client.util.Key
      private java.lang.String userKey;

      /** Email or immutable Id of the user as admin
       */
      public java.lang.String getUserKey() {
        return userKey;
      }

      /** Email or immutable Id of the user as admin */
      public MakeAdmin setUserKey(java.lang.String userKey) {
        this.userKey = userKey;
        return this;
      }

      @Override
      public MakeAdmin set(String parameterName, Object value) {
        return (MakeAdmin) super.set(parameterName, value);
      }
    }
    /**
     * update user. This method supports patch semantics.
     *
     * Create a request for the method "users.patch".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
     *
     * @param userKey Email or immutable Id of the user. If Id, it should match with id of user object
     * @param content the {@link com.google.api.services.admin.directory.model.User}
     * @return the request
     */
    public Patch patch(java.lang.String userKey, com.google.api.services.admin.directory.model.User content) throws java.io.IOException {
      Patch result = new Patch(userKey, content);
      initialize(result);
      return result;
    }

    public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.User> {

      private static final String REST_PATH = "users/{userKey}";

      /**
       * update user. This method supports patch semantics.
       *
       * Create a request for the method "users.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
       * Patch#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param userKey Email or immutable Id of the user. If Id, it should match with id of user object
       * @param content the {@link com.google.api.services.admin.directory.model.User}
       * @since 1.13
       */
      protected Patch(java.lang.String userKey, com.google.api.services.admin.directory.model.User content) {
        super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.User.class);
        this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public Patch setAlt(java.lang.String alt) {
        return (Patch) super.setAlt(alt);
      }

      @Override
      public Patch setFields(java.lang.String fields) {
        return (Patch) super.setFields(fields);
      }

      @Override
      public Patch setKey(java.lang.String key) {
        return (Patch) super.setKey(key);
      }

      @Override
      public Patch setOauthToken(java.lang.String oauthToken) {
        return (Patch) super.setOauthToken(oauthToken);
      }

      @Override
      public Patch setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Patch) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Patch setQuotaUser(java.lang.String quotaUser) {
        return (Patch) super.setQuotaUser(quotaUser);
      }

      @Override
      public Patch setUserIp(java.lang.String userIp) {
        return (Patch) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the user. If Id, it should match with id of user object */
      @com.google.api.client.util.Key
      private java.lang.String userKey;

      /** Email or immutable Id of the user. If Id, it should match with id of user object
       */
      public java.lang.String getUserKey() {
        return userKey;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of user object */
      public Patch setUserKey(java.lang.String userKey) {
        this.userKey = userKey;
        return this;
      }

      @Override
      public Patch set(String parameterName, Object value) {
        return (Patch) super.set(parameterName, value);
      }
    }
    /**
     * Undelete a deleted user
     *
     * Create a request for the method "users.undelete".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Undelete#execute()} method to invoke the remote operation.
     *
     * @param userKey The immutable id of the user
     * @param content the {@link com.google.api.services.admin.directory.model.UserUndelete}
     * @return the request
     */
    public Undelete undelete(java.lang.String userKey, com.google.api.services.admin.directory.model.UserUndelete content) throws java.io.IOException {
      Undelete result = new Undelete(userKey, content);
      initialize(result);
      return result;
    }

    public class Undelete extends DirectoryRequest<Void> {

      private static final String REST_PATH = "users/{userKey}/undelete";

      /**
       * Undelete a deleted user
       *
       * Create a request for the method "users.undelete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Undelete#execute()} method to invoke the remote operation. <p>
       * {@link
       * Undelete#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)}
       * must be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param userKey The immutable id of the user
       * @param content the {@link com.google.api.services.admin.directory.model.UserUndelete}
       * @since 1.13
       */
      protected Undelete(java.lang.String userKey, com.google.api.services.admin.directory.model.UserUndelete content) {
        super(Directory.this, "POST", REST_PATH, content, Void.class);
        this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public Undelete setAlt(java.lang.String alt) {
        return (Undelete) super.setAlt(alt);
      }

      @Override
      public Undelete setFields(java.lang.String fields) {
        return (Undelete) super.setFields(fields);
      }

      @Override
      public Undelete setKey(java.lang.String key) {
        return (Undelete) super.setKey(key);
      }

      @Override
      public Undelete setOauthToken(java.lang.String oauthToken) {
        return (Undelete) super.setOauthToken(oauthToken);
      }

      @Override
      public Undelete setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Undelete) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Undelete setQuotaUser(java.lang.String quotaUser) {
        return (Undelete) super.setQuotaUser(quotaUser);
      }

      @Override
      public Undelete setUserIp(java.lang.String userIp) {
        return (Undelete) super.setUserIp(userIp);
      }

      /** The immutable id of the user */
      @com.google.api.client.util.Key
      private java.lang.String userKey;

      /** The immutable id of the user
       */
      public java.lang.String getUserKey() {
        return userKey;
      }

      /** The immutable id of the user */
      public Undelete setUserKey(java.lang.String userKey) {
        this.userKey = userKey;
        return this;
      }

      @Override
      public Undelete set(String parameterName, Object value) {
        return (Undelete) super.set(parameterName, value);
      }
    }
    /**
     * update user
     *
     * Create a request for the method "users.update".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Update#execute()} method to invoke the remote operation.
     *
     * @param userKey Email or immutable Id of the user. If Id, it should match with id of user object
     * @param content the {@link com.google.api.services.admin.directory.model.User}
     * @return the request
     */
    public Update update(java.lang.String userKey, com.google.api.services.admin.directory.model.User content) throws java.io.IOException {
      Update result = new Update(userKey, content);
      initialize(result);
      return result;
    }

    public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.User> {

      private static final String REST_PATH = "users/{userKey}";

      /**
       * update user
       *
       * Create a request for the method "users.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
       * Update#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
       * be called to initialize this instance immediately after invoking the constructor. </p>
       *
       * @param userKey Email or immutable Id of the user. If Id, it should match with id of user object
       * @param content the {@link com.google.api.services.admin.directory.model.User}
       * @since 1.13
       */
      protected Update(java.lang.String userKey, com.google.api.services.admin.directory.model.User content) {
        super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.User.class);
        this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
      }

      @Override
      public Update setAlt(java.lang.String alt) {
        return (Update) super.setAlt(alt);
      }

      @Override
      public Update setFields(java.lang.String fields) {
        return (Update) super.setFields(fields);
      }

      @Override
      public Update setKey(java.lang.String key) {
        return (Update) super.setKey(key);
      }

      @Override
      public Update setOauthToken(java.lang.String oauthToken) {
        return (Update) super.setOauthToken(oauthToken);
      }

      @Override
      public Update setPrettyPrint(java.lang.Boolean prettyPrint) {
        return (Update) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Update setQuotaUser(java.lang.String quotaUser) {
        return (Update) super.setQuotaUser(quotaUser);
      }

      @Override
      public Update setUserIp(java.lang.String userIp) {
        return (Update) super.setUserIp(userIp);
      }

      /** Email or immutable Id of the user. If Id, it should match with id of user object */
      @com.google.api.client.util.Key
      private java.lang.String userKey;

      /** Email or immutable Id of the user. If Id, it should match with id of user object
       */
      public java.lang.String getUserKey() {
        return userKey;
      }

      /** Email or immutable Id of the user. If Id, it should match with id of user object */
      public Update setUserKey(java.lang.String userKey) {
        this.userKey = userKey;
        return this;
      }

      @Override
      public Update set(String parameterName, Object value) {
        return (Update) super.set(parameterName, value);
      }
    }

    /**
     * An accessor for creating requests from the Aliases collection.
     *
     * <p>The typical use is:</p>
     * <pre>
     *   {@code Directory admin = new Directory(...);}
     *   {@code Directory.Aliases.List request = admin.aliases().list(parameters ...)}
     * </pre>
     *
     * @return the resource collection
     */
    public Aliases aliases() {
      return new Aliases();
    }

    /**
     * The "aliases" collection of methods.
     */
    public class Aliases {

      /**
       * Remove a alias for the user
       *
       * Create a request for the method "aliases.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @param alias The alias to be removed
       * @return the request
       */
      public Delete delete(java.lang.String userKey, java.lang.String alias) throws java.io.IOException {
        Delete result = new Delete(userKey, alias);
        initialize(result);
        return result;
      }

      public class Delete extends DirectoryRequest<Void> {

        private static final String REST_PATH = "users/{userKey}/aliases/{alias}";

        /**
         * Remove a alias for the user
         *
         * Create a request for the method "aliases.delete".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
         * Delete#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
         * be called to initialize this instance immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @param alias The alias to be removed
         * @since 1.13
         */
        protected Delete(java.lang.String userKey, java.lang.String alias) {
          super(Directory.this, "DELETE", REST_PATH, null, Void.class);
          this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
          this.alias = com.google.api.client.util.Preconditions.checkNotNull(alias, "Required parameter alias must be specified.");
        }

        @Override
        public Delete setAlt(java.lang.String alt) {
          return (Delete) super.setAlt(alt);
        }

        @Override
        public Delete setFields(java.lang.String fields) {
          return (Delete) super.setFields(fields);
        }

        @Override
        public Delete setKey(java.lang.String key) {
          return (Delete) super.setKey(key);
        }

        @Override
        public Delete setOauthToken(java.lang.String oauthToken) {
          return (Delete) super.setOauthToken(oauthToken);
        }

        @Override
        public Delete setPrettyPrint(java.lang.Boolean prettyPrint) {
          return (Delete) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Delete setQuotaUser(java.lang.String quotaUser) {
          return (Delete) super.setQuotaUser(quotaUser);
        }

        @Override
        public Delete setUserIp(java.lang.String userIp) {
          return (Delete) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private java.lang.String userKey;

        /** Email or immutable Id of the user
         */
        public java.lang.String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Delete setUserKey(java.lang.String userKey) {
          this.userKey = userKey;
          return this;
        }

        /** The alias to be removed */
        @com.google.api.client.util.Key
        private java.lang.String alias;

        /** The alias to be removed
         */
        public java.lang.String getAlias() {
          return alias;
        }

        /** The alias to be removed */
        public Delete setAlias(java.lang.String alias) {
          this.alias = alias;
          return this;
        }

        @Override
        public Delete set(String parameterName, Object value) {
          return (Delete) super.set(parameterName, value);
        }
      }
      /**
       * Add a alias for the user
       *
       * Create a request for the method "aliases.insert".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Insert#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @param content the {@link com.google.api.services.admin.directory.model.Alias}
       * @return the request
       */
      public Insert insert(java.lang.String userKey, com.google.api.services.admin.directory.model.Alias content) throws java.io.IOException {
        Insert result = new Insert(userKey, content);
        initialize(result);
        return result;
      }

      public class Insert extends DirectoryRequest<com.google.api.services.admin.directory.model.Alias> {

        private static final String REST_PATH = "users/{userKey}/aliases";

        /**
         * Add a alias for the user
         *
         * Create a request for the method "aliases.insert".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Insert#execute()} method to invoke the remote operation. <p> {@link
         * Insert#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
         * be called to initialize this instance immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @param content the {@link com.google.api.services.admin.directory.model.Alias}
         * @since 1.13
         */
        protected Insert(java.lang.String userKey, com.google.api.services.admin.directory.model.Alias content) {
          super(Directory.this, "POST", REST_PATH, content, com.google.api.services.admin.directory.model.Alias.class);
          this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        }

        @Override
        public Insert setAlt(java.lang.String alt) {
          return (Insert) super.setAlt(alt);
        }

        @Override
        public Insert setFields(java.lang.String fields) {
          return (Insert) super.setFields(fields);
        }

        @Override
        public Insert setKey(java.lang.String key) {
          return (Insert) super.setKey(key);
        }

        @Override
        public Insert setOauthToken(java.lang.String oauthToken) {
          return (Insert) super.setOauthToken(oauthToken);
        }

        @Override
        public Insert setPrettyPrint(java.lang.Boolean prettyPrint) {
          return (Insert) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Insert setQuotaUser(java.lang.String quotaUser) {
          return (Insert) super.setQuotaUser(quotaUser);
        }

        @Override
        public Insert setUserIp(java.lang.String userIp) {
          return (Insert) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private java.lang.String userKey;

        /** Email or immutable Id of the user
         */
        public java.lang.String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Insert setUserKey(java.lang.String userKey) {
          this.userKey = userKey;
          return this;
        }

        @Override
        public Insert set(String parameterName, Object value) {
          return (Insert) super.set(parameterName, value);
        }
      }
      /**
       * List all aliases for a user
       *
       * Create a request for the method "aliases.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @return the request
       */
      public List list(java.lang.String userKey) throws java.io.IOException {
        List result = new List(userKey);
        initialize(result);
        return result;
      }

      public class List extends DirectoryRequest<com.google.api.services.admin.directory.model.Aliases> {

        private static final String REST_PATH = "users/{userKey}/aliases";

        /**
         * List all aliases for a user
         *
         * Create a request for the method "aliases.list".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
         * List#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
         * called to initialize this instance immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @since 1.13
         */
        protected List(java.lang.String userKey) {
          super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.Aliases.class);
          this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        }

        @Override
        public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
          return super.executeUsingHead();
        }

        @Override
        public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
          return super.buildHttpRequestUsingHead();
        }

        @Override
        public List setAlt(java.lang.String alt) {
          return (List) super.setAlt(alt);
        }

        @Override
        public List setFields(java.lang.String fields) {
          return (List) super.setFields(fields);
        }

        @Override
        public List setKey(java.lang.String key) {
          return (List) super.setKey(key);
        }

        @Override
        public List setOauthToken(java.lang.String oauthToken) {
          return (List) super.setOauthToken(oauthToken);
        }

        @Override
        public List setPrettyPrint(java.lang.Boolean prettyPrint) {
          return (List) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public List setQuotaUser(java.lang.String quotaUser) {
          return (List) super.setQuotaUser(quotaUser);
        }

        @Override
        public List setUserIp(java.lang.String userIp) {
          return (List) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private java.lang.String userKey;

        /** Email or immutable Id of the user
         */
        public java.lang.String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public List setUserKey(java.lang.String userKey) {
          this.userKey = userKey;
          return this;
        }

        @Override
        public List set(String parameterName, Object value) {
          return (List) super.set(parameterName, value);
        }
      }

    }
    /**
     * An accessor for creating requests from the Photos collection.
     *
     * <p>The typical use is:</p>
     * <pre>
     *   {@code Directory admin = new Directory(...);}
     *   {@code Directory.Photos.List request = admin.photos().list(parameters ...)}
     * </pre>
     *
     * @return the resource collection
     */
    public Photos photos() {
      return new Photos();
    }

    /**
     * The "photos" collection of methods.
     */
    public class Photos {

      /**
       * Remove photos for the user
       *
       * Create a request for the method "photos.delete".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Delete#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @return the request
       */
      public Delete delete(java.lang.String userKey) throws java.io.IOException {
        Delete result = new Delete(userKey);
        initialize(result);
        return result;
      }

      public class Delete extends DirectoryRequest<Void> {

        private static final String REST_PATH = "users/{userKey}/photos/thumbnail";

        /**
         * Remove photos for the user
         *
         * Create a request for the method "photos.delete".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Delete#execute()} method to invoke the remote operation. <p> {@link
         * Delete#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
         * be called to initialize this instance immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @since 1.13
         */
        protected Delete(java.lang.String userKey) {
          super(Directory.this, "DELETE", REST_PATH, null, Void.class);
          this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        }

        @Override
        public Delete setAlt(java.lang.String alt) {
          return (Delete) super.setAlt(alt);
        }

        @Override
        public Delete setFields(java.lang.String fields) {
          return (Delete) super.setFields(fields);
        }

        @Override
        public Delete setKey(java.lang.String key) {
          return (Delete) super.setKey(key);
        }

        @Override
        public Delete setOauthToken(java.lang.String oauthToken) {
          return (Delete) super.setOauthToken(oauthToken);
        }

        @Override
        public Delete setPrettyPrint(java.lang.Boolean prettyPrint) {
          return (Delete) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Delete setQuotaUser(java.lang.String quotaUser) {
          return (Delete) super.setQuotaUser(quotaUser);
        }

        @Override
        public Delete setUserIp(java.lang.String userIp) {
          return (Delete) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private java.lang.String userKey;

        /** Email or immutable Id of the user
         */
        public java.lang.String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Delete setUserKey(java.lang.String userKey) {
          this.userKey = userKey;
          return this;
        }

        @Override
        public Delete set(String parameterName, Object value) {
          return (Delete) super.set(parameterName, value);
        }
      }
      /**
       * Retrieve photo of a user
       *
       * Create a request for the method "photos.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @return the request
       */
      public Get get(java.lang.String userKey) throws java.io.IOException {
        Get result = new Get(userKey);
        initialize(result);
        return result;
      }

      public class Get extends DirectoryRequest<com.google.api.services.admin.directory.model.UserPhoto> {

        private static final String REST_PATH = "users/{userKey}/photos/thumbnail";

        /**
         * Retrieve photo of a user
         *
         * Create a request for the method "photos.get".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
         * Get#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must be
         * called to initialize this instance immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @since 1.13
         */
        protected Get(java.lang.String userKey) {
          super(Directory.this, "GET", REST_PATH, null, com.google.api.services.admin.directory.model.UserPhoto.class);
          this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        }

        @Override
        public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
          return super.executeUsingHead();
        }

        @Override
        public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
          return super.buildHttpRequestUsingHead();
        }

        @Override
        public Get setAlt(java.lang.String alt) {
          return (Get) super.setAlt(alt);
        }

        @Override
        public Get setFields(java.lang.String fields) {
          return (Get) super.setFields(fields);
        }

        @Override
        public Get setKey(java.lang.String key) {
          return (Get) super.setKey(key);
        }

        @Override
        public Get setOauthToken(java.lang.String oauthToken) {
          return (Get) super.setOauthToken(oauthToken);
        }

        @Override
        public Get setPrettyPrint(java.lang.Boolean prettyPrint) {
          return (Get) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Get setQuotaUser(java.lang.String quotaUser) {
          return (Get) super.setQuotaUser(quotaUser);
        }

        @Override
        public Get setUserIp(java.lang.String userIp) {
          return (Get) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private java.lang.String userKey;

        /** Email or immutable Id of the user
         */
        public java.lang.String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Get setUserKey(java.lang.String userKey) {
          this.userKey = userKey;
          return this;
        }

        @Override
        public Get set(String parameterName, Object value) {
          return (Get) super.set(parameterName, value);
        }
      }
      /**
       * Add a photo for the user. This method supports patch semantics.
       *
       * Create a request for the method "photos.patch".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Patch#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @param content the {@link com.google.api.services.admin.directory.model.UserPhoto}
       * @return the request
       */
      public Patch patch(java.lang.String userKey, com.google.api.services.admin.directory.model.UserPhoto content) throws java.io.IOException {
        Patch result = new Patch(userKey, content);
        initialize(result);
        return result;
      }

      public class Patch extends DirectoryRequest<com.google.api.services.admin.directory.model.UserPhoto> {

        private static final String REST_PATH = "users/{userKey}/photos/thumbnail";

        /**
         * Add a photo for the user. This method supports patch semantics.
         *
         * Create a request for the method "photos.patch".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Patch#execute()} method to invoke the remote operation. <p> {@link
         * Patch#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
         * be called to initialize this instance immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @param content the {@link com.google.api.services.admin.directory.model.UserPhoto}
         * @since 1.13
         */
        protected Patch(java.lang.String userKey, com.google.api.services.admin.directory.model.UserPhoto content) {
          super(Directory.this, "PATCH", REST_PATH, content, com.google.api.services.admin.directory.model.UserPhoto.class);
          this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        }

        @Override
        public Patch setAlt(java.lang.String alt) {
          return (Patch) super.setAlt(alt);
        }

        @Override
        public Patch setFields(java.lang.String fields) {
          return (Patch) super.setFields(fields);
        }

        @Override
        public Patch setKey(java.lang.String key) {
          return (Patch) super.setKey(key);
        }

        @Override
        public Patch setOauthToken(java.lang.String oauthToken) {
          return (Patch) super.setOauthToken(oauthToken);
        }

        @Override
        public Patch setPrettyPrint(java.lang.Boolean prettyPrint) {
          return (Patch) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Patch setQuotaUser(java.lang.String quotaUser) {
          return (Patch) super.setQuotaUser(quotaUser);
        }

        @Override
        public Patch setUserIp(java.lang.String userIp) {
          return (Patch) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private java.lang.String userKey;

        /** Email or immutable Id of the user
         */
        public java.lang.String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Patch setUserKey(java.lang.String userKey) {
          this.userKey = userKey;
          return this;
        }

        @Override
        public Patch set(String parameterName, Object value) {
          return (Patch) super.set(parameterName, value);
        }
      }
      /**
       * Add a photo for the user
       *
       * Create a request for the method "photos.update".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Update#execute()} method to invoke the remote operation.
       *
       * @param userKey Email or immutable Id of the user
       * @param content the {@link com.google.api.services.admin.directory.model.UserPhoto}
       * @return the request
       */
      public Update update(java.lang.String userKey, com.google.api.services.admin.directory.model.UserPhoto content) throws java.io.IOException {
        Update result = new Update(userKey, content);
        initialize(result);
        return result;
      }

      public class Update extends DirectoryRequest<com.google.api.services.admin.directory.model.UserPhoto> {

        private static final String REST_PATH = "users/{userKey}/photos/thumbnail";

        /**
         * Add a photo for the user
         *
         * Create a request for the method "photos.update".
         *
         * This request holds the parameters needed by the the admin server.  After setting any optional
         * parameters, call the {@link Update#execute()} method to invoke the remote operation. <p> {@link
         * Update#initialize(com.google.api.client.googleapis.services.AbstractGoogleClientRequest)} must
         * be called to initialize this instance immediately after invoking the constructor. </p>
         *
         * @param userKey Email or immutable Id of the user
         * @param content the {@link com.google.api.services.admin.directory.model.UserPhoto}
         * @since 1.13
         */
        protected Update(java.lang.String userKey, com.google.api.services.admin.directory.model.UserPhoto content) {
          super(Directory.this, "PUT", REST_PATH, content, com.google.api.services.admin.directory.model.UserPhoto.class);
          this.userKey = com.google.api.client.util.Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
          checkRequiredParameter(content, "content");
          checkRequiredParameter(content.getPhotoData(), "UserPhoto.getPhotoData()");
        }

        @Override
        public Update setAlt(java.lang.String alt) {
          return (Update) super.setAlt(alt);
        }

        @Override
        public Update setFields(java.lang.String fields) {
          return (Update) super.setFields(fields);
        }

        @Override
        public Update setKey(java.lang.String key) {
          return (Update) super.setKey(key);
        }

        @Override
        public Update setOauthToken(java.lang.String oauthToken) {
          return (Update) super.setOauthToken(oauthToken);
        }

        @Override
        public Update setPrettyPrint(java.lang.Boolean prettyPrint) {
          return (Update) super.setPrettyPrint(prettyPrint);
        }

        @Override
        public Update setQuotaUser(java.lang.String quotaUser) {
          return (Update) super.setQuotaUser(quotaUser);
        }

        @Override
        public Update setUserIp(java.lang.String userIp) {
          return (Update) super.setUserIp(userIp);
        }

        /** Email or immutable Id of the user */
        @com.google.api.client.util.Key
        private java.lang.String userKey;

        /** Email or immutable Id of the user
         */
        public java.lang.String getUserKey() {
          return userKey;
        }

        /** Email or immutable Id of the user */
        public Update setUserKey(java.lang.String userKey) {
          this.userKey = userKey;
          return this;
        }

        @Override
        public Update set(String parameterName, Object value) {
          return (Update) super.set(parameterName, value);
        }
      }

    }
  }

  /**
   * Builder for {@link Directory}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport, which should normally be:
     *        <ul>
     *        <li>Google App Engine:
     *        {@code com.google.api.client.extensions.appengine.http.UrlFetchTransport}</li>
     *        <li>Android: {@code newCompatibleTransport} from
     *        {@code com.google.api.client.extensions.android.http.AndroidHttp}</li>
     *        <li>Java: {@link com.google.api.client.googleapis.javanet.GoogleNetHttpTransport#newTrustedTransport()}
     *        </li>
     *        </ul>
     * @param jsonFactory JSON factory, which may be:
     *        <ul>
     *        <li>Jackson: {@code com.google.api.client.json.jackson2.JacksonFactory}</li>
     *        <li>Google GSON: {@code com.google.api.client.json.gson.GsonFactory}</li>
     *        <li>Android Honeycomb or higher:
     *        {@code com.google.api.client.extensions.android.json.AndroidJsonFactory}</li>
     *        </ul>
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(com.google.api.client.http.HttpTransport transport, com.google.api.client.json.JsonFactory jsonFactory,
        com.google.api.client.http.HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Directory}. */
    @Override
    public Directory build() {
      return new Directory(this);
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(com.google.api.client.http.HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    @Override
    public Builder setSuppressRequiredParameterChecks(boolean suppressRequiredParameterChecks) {
      return (Builder) super.setSuppressRequiredParameterChecks(suppressRequiredParameterChecks);
    }

    @Override
    public Builder setSuppressAllChecks(boolean suppressAllChecks) {
      return (Builder) super.setSuppressAllChecks(suppressAllChecks);
    }

    /**
     * Set the {@link DirectoryRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setDirectoryRequestInitializer(
        DirectoryRequestInitializer directoryRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(directoryRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        com.google.api.client.googleapis.services.GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
